# -*- coding: utf-8 -*-
#
# Copyright (c) 2006 by BlueDynamics Alliance, Klein & Partner KEG, Austria
#
# GNU General Public License (GPL)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#
__author__ = """Jens W. Klein <jens@bluedynamics.com>"""
__docformat__ = 'plaintext'

from Acquisition import aq_inner
from Acquisition import aq_parent
from Products.Five import BrowserView 
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.utils import _createObjectByType 
from Products.CMFPlone.interfaces.Translatable import ITranslatable

class LinguaLinkPortlet(BrowserView):
    
    def __init__(self, context, request):
        self.context = context
        self.request = request
        self.status = ''
    
    
    def performActions(self):
        """check request and do whats needed."""
        if self.request.form.get('createLinguaLinks', None) == 'allmissing':
            self.createLinguaLinks()
    
    def getUnprocessedLanguages(self):
        """returns list of lang-codes with all untranslated and not linked."""
        # fetch is-state
        langtool = self.context.portal_languages
        supported = langtool.getSupportedLanguages()
        
        canonical = self.context.getCanonical()
        trans = canonical.getTranslationLanguages()

        # prune list of untranslated
        llinks = self.getLinguaLinks()
        unprocessed = [lang for lang in supported if lang not in trans]
        unprocessed = [lang for lang in unprocessed if lang != canonical.Language()]  
        unprocessed = [lang for lang in unprocessed if lang not in llinks.keys()]
        return unprocessed
            
    def createLinguaLinks(self):
        """Creates LinguaLinks for the context."""
        
        langtool = self.context.portal_languages

        # fetch canonical
        canonical = self.context.getCanonical()

        # enforce canonical to have a language
        canonicallang = canonical.Language()
        if not canonicallang:
            canonicallang = langtool.getDefaultLanguage()
            canonical.setLanguage(canonicallang)
            canonical.reindexObject()        

        unprocessed = self.getUnprocessedLanguages()
        
        if not unprocessed:
            self.context.plone_utils.addPortalMessage("Tried to create " + \
                "LinguaLinks, but content is complete translated and linked.")
            return 
        
        for lang in unprocessed:
            parent = aq_parent(aq_inner(canonical))
            if ITranslatable.isImplementedBy(parent):
                parent = parent.getTranslation(lang) or parent
            id = canonical.getId()
            while not parent.checkIdAvailable(id):
                id = '%s-%s' % (id, lang)                
            _createObjectByType('LinguaLink', parent, id)
            ll = parent[id]
            ll.setTitle(canonical.Title())
            ll.setDescription(canonical.Description())
            ll.setLinguaLink(canonical, lang)
            ll.reindexObject()
            self.context.plone_utils.addPortalMessage("Added LinguaLink for language '%s' at '%s/%s'." % (lang, parent.absolute_url(), id))     
    
    def isLinguaLink(self):
        """check if the context is a LinguaLink.""" 
        return self.context.getPortalTypeName() == 'LinguaLink'       
        
    def isTranslatable(self):
        """check if the context is translateable."""
        return ITranslatable.isImplementedBy(self.context)
        
    def hasTranslations(self):
        """check if context is already translated."""
        if self.context.isCanonical():
            trans = self.context.getNonCanonicalTranslations()
            if len(trans):
                return True
        return False
    
    def isTranslatedToAllLanguages(self):
        """check if translations are complete."""
        if not self.isTranslatable():
            return True
        
        trans = self.context.getTranslationLanguages()
        langtool = getToolByName(self, 'portal_languages', None)
        if langtool is None:
            return True
        
        supported = langtool.getSupportedLanguages()
        if len(trans) >= len(supported):
            return True
        
        return False

    def getLinguaLinks(self):
        """returns all LinguaLinks."""
        canonical = self.context.getCanonical()
        if not self.isTranslatable():
            return {}
        llinks = canonical.getBRefs('lingualink')
        langlinkdict = {}
        #import pdb; pdb.set_trace()
        dummy = [langlinkdict.update({ll.Language(): ll}) for ll in llinks]
        return langlinkdict
        